<?php
/**
 * Menu functions here to keep the global admin helper file as small as possible
 * This file is loaded only in i-Search menu
 *
 * @since 3.1.0
 *
 */

/**
 * Render the menu html for the instance builder.
 *
 * @param array $settings
 * @param bool  $is_new
 */
function isrc_render_admin_filter_groups_html( $settings = array(), $is_new = false ) {

	if ( $is_new ) {
		$settings['flt']['GRKEY'] = array(
			'gr_l' => 'Group Title',
			'fts'  => array(
				array(
					'title' => 'Filter Title',
					'def'   => 'All',
					'apper' => 'dropdown',
					'src'   => ''
				)
			)
		);
	}

	foreach ( $settings['flt'] as $grkey => $grval ) {

		?>
        <li class="isrc-filter-group-header-wrap closest-group" data-groupno="<?php echo ( ! $is_new ) ? $grkey : 'GRKEY'; ?>">

            <div class="isrc-filter-group-header-col">
                <span class="noholder"><?php echo ( ! $is_new ) ? $grkey + 1 : 'GROUPKEYNO'; ?></span>
                <span class="isrc-filter-group-title isrc-filter-group-label-dest">
                <?php
                if ( isset( $grval['gr_l'] ) ) {
	                echo $grval['gr_l'];
                } else {
	                echo '';
                }
                ?>
            </span>
                <div class="isrc-filter-rm-group"><?php _e( 'Remove', 'i_search' ); ?></div>
            </div>
            <div class="isrc-filter-settings-section-header">
                <div class="isrc-field isrc-settings">
                    <div class="isrc-label">
                        <label>
							<?php _e( 'Group Label', 'i_search' ); ?>
                            <p class="description">
								<?php _e( 'The visible label of this group.', 'i_search' ); ?>
                            </p>
                        </label>
                    </div>
                    <div class="isrc-input">
						<?php
						isrc_render_text_field(
							'isrc_sc_opt[flt][' . $grkey . '][gr_l]',
							'',
							( isset( $settings['flt'][ $grkey ]['gr_l'] ) ) ? $settings['flt'][ $grkey ]['gr_l'] : '',
							'',
							'',
							'isrc-filter-group-label-source'
						);
						?>
                    </div>
                    <div class="clear"></div>
                </div>
                <div class="isrc-field isrc-settings">
                    <div class="isrc-label">
                        <label>
				            <?php _e( 'Display group label in frontend?', 'i_search' ); ?>
                        </label>
                    </div>
                    <div class="isrc-input">
			            <?php
			            isrc_render_switch_html(
				            '',
				            'isrc_sc_opt[flt][' . $grkey . '][gr_l_ed]',
				            'Yes',
				            'No',
				            '',
				            ( isset( $settings['flt'][ $grkey ]['gr_l_ed'] ) ),
				            $is_new,
				            true
			            );
			            ?>
                    </div>
                    <div class="clear"></div>
                </div>
                <div class="isrc-field isrc-settings">
                    <div class="isrc-label">
                        <label>
							<?php _e( 'Group CSS Class', 'i_search' ); ?>
                            <p class="description">
								<?php _e( 'Optional. Add a css class.', 'i_search' ); ?>
                            </p>
                        </label>
                    </div>
                    <div class="isrc-input">
						<?php
						isrc_render_text_field(
							'isrc_sc_opt[flt][' . $grkey . '][css_c]',
							'',
							( isset( $settings['flt'][ $grkey ]['css_c'] ) ) ? $settings['flt'][ $grkey ]['css_c'] : '',
							'',
							'',
							'isrc-filter-group-css_c-source'
						);
						?>
                    </div>
                    <div class="clear"></div>
                </div>
                <div class="isrc-field isrc-settings">
                    <div class="isrc-label">
                        <label>
							<?php _e( 'Enable this filter group?', 'i_search' ); ?>
                        </label>
                    </div>
                    <div class="isrc-input">
						<?php
						isrc_render_switch_html(
							'',
							'isrc_sc_opt[flt][' . $grkey . '][gr_ed]',
							'Yes',
							'No',
							'',
							( isset( $settings['flt'][ $grkey ]['gr_ed'] ) ),
							$is_new,
							true
						);
						?>
                    </div>
                    <div class="clear"></div>
                </div>
                <div class="isrc-field isrc-settings">
                    <div class="isrc-label">
                        <label>
							<?php _e( 'Filters in Group', 'i_search' ); ?>
                            <p class="description">
								<?php _e( 'Add filters here.', 'i_search' ); ?>
                            </p>
                        </label>
                    </div>
                    <ul class="isrc-input isrc-filter-append">
                        <!-- Start Repeater -->

						<?php

						if ( isset( $grval['fts'] ) && is_array( $grval['fts'] ) ) {
							foreach ( $grval['fts'] as $ftkey => $ftval ) {
								isrc_get_filter_in_group( $is_new, $grkey, $ftkey, $title = $ftval['title'], $ftval['def'], $apper = $ftval['apper'], $src = $ftval['src'], isset( $ftval['src_key'] ) ? $ftval['src_key'] : null, $settings );
							}
						}
						?>
                        <!-- End Repeater -->

                    </ul>
                    <div class="isrc-btn-v3 isrc-btn-v3-black isrc-setup-filters addNewFilterInGroupBtn"><i
                                class="fas fa-plus"></i><?php _e( 'Add New Filter into Group', 'i_Search' ); ?></div>
                    <div class="clear"></div>
                </div>

            </div>
        </li>
		<?php
	}
}

function isrc_get_filter_in_group( $is_new = false, $grkey = 0, $ftkey = 0, $title = 'Title', $def = '', $apper = '', $src = '', $src_key = '', $settings = array() ) {
	if ( isset( $settings['flt'][ $grkey ]['fts'][ $ftkey ] ) ) {
		$data = $settings['flt'][ $grkey ]['fts'][ $ftkey ];
	} else {
		$data = array();
	}
	?>
    <li class="closest-filters" data-groupno="<?php echo $grkey; ?>" data-filterno="<?php echo $ftkey; ?>">
        <div class="sttngs-cont cnt-white-shadow">

            <div class="isrc-inside">

                <!-- Repeater Header -->
                <div class="isrc-filter-filter-header-col">
                    <span class="noholder"><?php echo ( ! $is_new ) ? $ftkey + 1 : 'FLTRCOUNTNO'; ?></span>
                    <span class="isrc-filter-filter-title isrc-filter-label-dest"><?php echo $title; ?></span>
                    <div class="isrc-filter-rm-filter"><?php _e( 'Remove', 'i_search' ); ?></div>

                </div>
                <div class="isrc-filter-settings-section">
                    <div class="isrc-preview-preloader" style="opacity: 0;display: none;"></div>
                    <!-- Repeater Content -->
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Filter Name', 'i_search' ); ?>
                                <p class="description">
									<?php _e( 'Not visible. Only for recognizing', 'i_search' ); ?>
                                </p>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_text_field(
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][title]',
								'',
								$title,
								'',
								'',
								'isrc-filter-label-source'
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Enable this filter?', 'i_search' ); ?>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_switch_html(
								'',
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][flt_ed]',
								'Yes',
								'No',
								'',
								( isset( $data['flt_ed'] ) ),
								$is_new,
								true
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Multiple selections?', 'i_search' ); ?>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_switch_html(
								'',
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][multiselect]',
								'Yes',
								'No',
								'',
								( isset( $data['multiselect'] ) ),
								$is_new,
								true
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Text before filter', 'i_search' ); ?>
                                <p class="description">
									<?php _e( 'Leave empty to hide', 'i_search' ); ?>
                                </p>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_text_field(
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][flttxt]',
								'',
								( ! empty( $data['flttxt'] ) ) ? $data['flttxt'] : '',
								'',
								'',
								''
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Label for default', 'i_search' ); ?>
                                <p class="description">
									<?php _e( 'Ex: Select Category', 'i_search' ); ?>
                                </p>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_text_field(
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][def]',
								'',
								$def,
								'',
								'',
								''
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>

                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Appearance', 'i_search' ); ?>
                                <p class="description">
									<?php _e( 'Select the appearance of this field', 'i_search' ); ?>
                                </p>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_select_field(
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][apper]',
								'',
								'isrc_appearance',
								'',
								'',
								'',
								array(
									'list'     => __( 'List style', 'i_search' ),
									'tagcloud' => __( 'Tag cloud', 'i_search' ),
								),
								$apper
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="filter_ajx_appearance_placeholder">
						<?php
						if ( isset( $settings['flt'][ $grkey ]['fts'][ $ftkey ]['apper'] ) ) {
							isrc_get_filter_sources_appearance( $settings['flt'][ $grkey ]['fts'][ $ftkey ]['apper'], $grkey, $ftkey, $settings );
						} elseif ( $is_new ) {
							isrc_get_filter_sources_appearance( 'list' );
						}
						?>
                    </div>
                    <div class="isrc-field isrc-settings">
                        <div class="isrc-label">
                            <label>
								<?php _e( 'Filter source', 'i_search' ); ?>
                            </label>
                        </div>
                        <div class="isrc-input">
							<?php
							isrc_render_select_field(
								'isrc_sc_opt[flt][' . $grkey . '][fts][' . $ftkey . '][src]',
								'',
								'filter_source',
								'',
								'',
								'',
								array(
									''   => __( 'Select Source', 'i_search' ),
									'tx' => __( 'Taxonomy (Tags)', 'i_search' ),
									'mk' => __( 'Metakey', 'i_search' ),
								),
								$src
							);
							?>
                        </div>
                        <div class="clear"></div>
                    </div>
                    <div class="filter_ajx_placeholder">
						<?php
						if ( isset( $src ) && ! empty( $src ) && isset( $src_key ) && ! empty( $src_key ) ) {
							isrc_get_filter_sources( $src, $grkey, $ftkey, $src_key, $settings['flt'][ $grkey ]['fts'][ $ftkey ] );
						}
						?>
                    </div>

                </div>
            </div>
        </div>
    </li>

	<?php
}

/**
 * Render the html field for filter appearance
 *
 * @param string $source
 * @param string $group_key
 * @param string $filter_key
 * @param array  $settings
 */
function isrc_get_filter_sources_appearance( $source = '', $group_key = 'GRKEY', $filter_key = 'FTKEY', $settings = array() ) {
	if ( isset( $settings['flt'][ $group_key ]['fts'][ $filter_key ]['appear']['data'] ) ) {
		$data = $settings['flt'][ $group_key ]['fts'][ $filter_key ]['appear']['data'];
	} else {
		$data = array();
	}
	if ( $source == 'list' || $source == 'tagcloud' ) {
		?>
        <div class="isrc-field ajx-loaded isrc-settings">
            <div class="isrc-label">
                <label><?php _e( 'Search field placeholder' ); ?>
                    <p class="description">
						<?php _e( 'If the items count is greater than 5, a little search field will appear.', 'i_search' ); ?>
                    </p>
                </label>
            </div>
            <div class="isrc-input">
				<?php
				isrc_render_text_field(
					'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][appear][data][ph]',
					'',
					( isset( $data['ph'] ) ) ? $data['ph'] : '',
					'',
					'',
					'',
					true,
					'text',
					'',
					'',
					'',
					''
				);
				?>
            </div>
            <div class="clear"></div>
        </div>
        <div class="isrc-field ajx-loaded isrc-settings">
            <div class="isrc-label">
                <label><?php _e( 'Disable search in filter?' ); ?>
                    <p class="description">
						<?php _e( 'If disabled all items will appear as a long list.', 'i_search' ); ?>
                    </p>
                </label>
            </div>
            <div class="isrc-input">
				<?php
				isrc_render_switch_html(
					'',
					'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][appear][data][ed_src]',
					'Yes',
					'No',
					'',
					( isset( $data['ed_src'] ) ),
					( isset( $data['ed_src'] ) ),
					true
				);
				?>
            </div>
            <div class="clear"></div>
        </div>
		<?php
	}
}

/**
 * Render the html field for filter sources
 *
 * @param string $source
 * @param string $group_key
 * @param string $filter_key
 * @param string $selected
 * @param array  $settings
 */
function isrc_get_filter_sources( $source = '', $group_key = 'GRKEY', $filter_key = 'FTKEY', $selected = '', $settings = array() ) {
	global $isrc_menu;

	if ( empty( $source ) ) {
		exit();
	}

	$source      = sanitize_text_field( $source );
	$selected    = sanitize_text_field( $selected );
	$selectfield = '';

	if ( $source == 'tx' ) {
		/* taxonomy */
		$output   = 'objects';
		$operator = 'and'; // 'and' or 'or'
		$results  = get_taxonomies( array(), $output, $operator );

		foreach ( $results as $key => $val ) {
			$selected_str = ( $selected == $val->name ) ? 'selected' : '';
			$selectfield  .= '<option ' . $selected_str . ' value="' . $val->name . '">' . $val->label . ' (' . $val->name . ')</option>';
		}
		$selectfield = '<select name="isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][src_key]" class="filter_source_key" style="">' . $selectfield . '</select>';
		?>
        <div class="isrc-field ajx-loaded isrc-settings">
            <div class="isrc-label">
                <label><?php _e( 'Source Taxonomy' ); ?> </label>
            </div>
            <div class="isrc-input">
				<?php echo $selectfield; ?>
            </div>
            <div class="clear"></div>
        </div>

        <div class="isrc-field isrc-settings">
            <div class="isrc-label">
                <label>
					<?php _e( 'Items in list', 'i_search' ); ?>
                </label>
            </div>
            <div class="isrc-input">
				<?php
				isrc_render_select_field(
					'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][items_list]',
					'',
					'items_cust_sel',
					'',
					'',
					'',
					array(
						'default' => __( 'Get all items from taxonomy source', 'i_search' ),
						'custom'  => __( 'Select some items from taxonomy source', 'i_search' ),
					),
					( ! empty( $settings ) ) ? $settings : 'default'
				);
				?>
            </div>
            <div class="clear"></div>
        </div>
		<?php
		$css_tags = 'isrc-hide';
		if ( isset( $settings['items_list'] ) && $settings['items_list'] == 'custom' ) {
			$css_tags = '';
		}
		?>
        <div class="isrc-field ajx-loaded isrc-settings ajax_placeholder_custom_tags <?php echo $css_tags; ?>">
            <div class="isrc-label">
                <label>
					<?php _e( 'Select items', 'i_search' ); ?>
                </label>
            </div>
            <ul class="isrc-input isrc-items-in-list">
				<?php isrc_flt_render_cust_tags_html( $group_key, $filter_key, $settings ); ?>
            </ul>
            <div class="clear"></div>
        </div>

		<?php

	} elseif ( $source == 'mk' ) {
		$meta_keys = $isrc_menu->get_the_meta_keys( true );
		foreach ( $meta_keys as $key => $val ) {
			$selected_str = ( $selected == $val['name'] ) ? 'selected' : '';
			$selectfield  .= '<option ' . $selected_str . ' value="' . $val['name'] . '">' . $val['label'] . '</option>';
		}
		$selectfield = '<select name="isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][src_key]" class="filter_source_key">' . $selectfield . '</select>';
		?>
        <div class="isrc-field ajx-loaded isrc-settings">
            <div class="isrc-label">
                <label><?php _e( 'Source Meta key' ); ?> </label>
            </div>
            <div class="isrc-input">
				<?php echo $selectfield; ?>
            </div>
            <div class="clear"></div>
        </div>

        <div class="isrc-field isrc-settings">
            <div class="isrc-label">
                <label>
					<?php _e( 'Items in list', 'i_search' ); ?>
                </label>
            </div>
            <div class="isrc-input">
				<?php
				isrc_render_select_field(
					'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][items_list]',
					'',
					'items_cust_sel',
					'',
					'',
					'',
					array(
						'default' => __( 'Get all items from meta key source', 'i_search' ),
						'custom'  => __( 'Select some items from meta key source', 'i_search' ),
					),
					( ! empty( $settings ) ) ? $settings : 'default'
				);
				?>
            </div>
            <div class="clear"></div>
        </div>
		<?php
		$css_tags = 'isrc-hide';
		if ( isset( $settings['items_list'] ) && $settings['items_list'] == 'custom' ) {
			$css_tags = '';
		}
		?>
        <div class="isrc-field ajx-loaded isrc-settings ajax_placeholder_custom_tags <?php echo $css_tags; ?>">
            <div class="isrc-label">
                <label>
					<?php _e( 'Select items', 'i_search' ); ?>
                </label>
            </div>
            <ul class="isrc-input isrc-items-in-list">
				<?php isrc_flt_render_cust_tags_html( $group_key, $filter_key, $settings ); ?>
            </ul>
            <div class="clear"></div>
        </div>

	<?php	}
}


/**
 * @param string $group_key
 * @param string $filter_key
 * @param array  $settings
 * @param string $item_key
 */
function isrc_flt_render_cust_tags_html( $group_key = 'GRKEY', $filter_key = 'FTKEY', $settings = array(), $item_key = 'ITKEY' ) {

	/* check if this is not ajaxing and we have already saved items */
	$items_in_list = array();
	if ( ! empty( $settings['items_in_list'] ) && is_array( $settings['items_in_list'] ) ) {
		$items_in_list = $settings['items_in_list'];
		foreach ( $items_in_list as $item_key => $val ) {
			isrc_flt_render_cust_tags_html_helper( $group_key, $filter_key, $item_key, array( $val['key'] => $val['key_txt'] ), $val['key_txt'], $val['key'], $val['label'] );
		}
	} else {
		isrc_flt_render_cust_tags_html_helper( $group_key, $filter_key, $item_key, '', '' );
	}
}

/**
 * Render the html field for custom keys
 *
 * @param string $group_key
 * @param string $filter_key
 * @param string $item_key
 * @param array  $select_html_values
 * @param string $hiddenkey_value
 * @param null   $selected_key
 * @param string $selected_label
 */
function isrc_flt_render_cust_tags_html_helper( $group_key = 'GRKEY', $filter_key = 'FTKEY', $item_key = 'ITKEY', $select_html_values = array(), $hiddenkey_value = '', $selected_key = null, $selected_label = '' ) {
	?>
    <li class="single_iscr_custom_tag_item">
        <div class="select2 cutom-flt-tag-select">
            <span class="ui-icon ui-icon-arrowthick-2-n-s"></span>
			<?php
			isrc_render_select_field(
				'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][items_in_list][' . $item_key . '][key]',
				'',
				'isrc_cust_item_key',
				'',
				'',
				'',
				$select_html_values,
				array( $selected_key )
			);
			?>
            <input name="<?php echo 'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][items_in_list][' . $item_key . '][key_txt]'; ?>"
                   type="hidden"
                   class="items_key_val_hidden"
                   value="<?php echo $hiddenkey_value; ?>">
        </div>
        <div class="cutom-flt-tag-inp">
            <span class="isrc_cust_item_label"><?php _e( 'Label:', 'i_search' ); ?></span>
			<?php
			isrc_render_text_field(
				'isrc_sc_opt[flt][' . $group_key . '][fts][' . $filter_key . '][items_in_list][' . $item_key . '][label]',
				'',
				$selected_label,
				'',
				'',
				'isrc_cust_item_label_inp',
				true,
				'text',
				'',
				'',
				'',
				''
			);
			?>
        </div>
        <div class="isrc-btns">
            <span class="isrc_cust_item_plus"><i class="fas fa-plus"></i></span>
            <span class="isrc_cust_item_minus"><i class="fas fa-minus"></i></span>
        </div>
    </li>

	<?php
}