<?php
// don't load directly
if (!defined('ABSPATH')) {
    die;
}

function isrc_filter_group_templates_html($group_val = array())
{
    if (empty($group_val)) {
        return false;
    }

    /* group enabled? */
    if (!isset($group_val['fts'])) {
        return false;
    }

    $group_filter_html = '';

    if (is_array($group_val['fts'])) {

        foreach ($group_val['fts'] as $fkey => $fval) {
            /* is filter disabled? */
            if (!isset($fval['flt_ed'])) {
                continue;
            }

            $filter_with_search = false;

            if (!isset($fval['apper']) || empty($fval['apper'])) {
                continue;
            }

            if (!isset($fval['src_key']) || empty($fval['src_key'])) {
                continue;
            }

            if (!isset($fval['src']) || empty($fval['src'])) {
                continue;
            }

            if (!isset($fval['items_list']) || empty($fval['items_list'])) {
                continue;
            }

            if ($fval['items_list'] == 'custom' && empty($fval['items_in_list'])) {
                continue;
            }

            $filter_html = isrc_filter_list_html($fkey, $fval, $filter_with_search);
            if ( $filter_html === false || empty( $filter_html ) ) {
                continue;
            }

            $group_filter_html .= $filter_html;
        }
    }

    if (empty($group_filter_html)) {
        return false;
    }


    return $group_filter_html;
}

function isc_render_term_li($terms, $fkey, $fval, &$into, $depth = 0)
{

    foreach ($terms as $key => $val) {
        $into .= '<li class="isrc-filter-item depth_' . $depth . '" data-parent="' . $val->parent . '" data-tx_id="' . $val->term_id . '" data-filter-text="' . wp_specialchars_decode($val->name) . '" data-selectable="yes" data-filter-id="' . $fkey . '" data-src="' . $fval['src'] . '" data-src_key="' . $fval['src_key'] . '" data-value="' . $val->term_id . '">';
        $into .= '<a class="isrc-dropdown-item-link"><span class="isrc-flt-checkbox-ui"></span><span class="iscr-flt-val">' . wp_specialchars_decode($val->name) . '</span></a></li>';
        if (isset($val->children) && !empty($val->children)) {
            isc_render_term_li($val->children, $fkey, $fval, $into, $depth + 1);
        }
    }
}

function isrc_filter_list_html($fkey = 0, $fval = array(), &$filter_with_search){

    if ($fval['src'] == 'tx') {
        /* taxonomy */

        $selectfield = '';

        /* if NOT custom keys selected in admin */
        if ($fval['items_list'] == 'default') {
            $terms = get_terms(array(
                'taxonomy' => $fval['src_key'],
                'hide_empty' => true,
            ));

            $categoryHierarchy = array();
            isrc_sort_terms_hierarchicaly($terms, $categoryHierarchy);
            $terms = $categoryHierarchy;
        } elseif ($fval['items_list'] == 'custom') {
            /* mime a WP_Term object to use the same functions. Build it. */

            $itemslist = array();
            $itemslist_obj = array();
            foreach ($fval['items_in_list'] as $itkey => $key) {
                $itemslist['term_id'] = $key['key'];
                $itemslist['parent'] = 0;
                $itemslist['children'] = array();
                $itemslist['name'] = (empty($key['label'])) ? wp_specialchars_decode($key['key_txt']) : wp_specialchars_decode($key['label']);
                $itemslist_obj[] = (object)$itemslist;
            }

            $terms = $itemslist_obj;
        }

        if (empty($terms)) {
            return false;
        }

        /* add js search */
        /* think reverse for enable disable mini search field. We are asking for disable search in the admin menu */
        if (count($terms) > 5 && !isset($fval['appear']['data']['ed_src'])) {

            if (isset($fval['appear']['data']['ph'])) {
                $placeholder = $fval['appear']['data']['ph'];
            } else {
                $placeholder = '';
            }

            $searchfield = '<input type="text" autocomplete="off" class="isrc-flt-src"  placeholder="' . $placeholder . '">';
            $filter_with_search = true;
        } else {
            $searchfield = '';
        }

        /* set select all with empty value */
        if (isset($fval['def']) && !empty($fval['def'])) {
            $selectfield .= '<li class="isrc-filter-item" data-empty="1" data-selectable="yes" data-filter-text="' . $fval['def'] . '" data-filter-id="' . $fkey . '" data-src="' . $fval['src'] . '" data-src_key="' . $fval['src_key'] . '" data-value="">';
            $selectfield .= '<a class="isrc-dropdown-item-link"><span class="isrc-flt-checkbox-ui"></span><span class="iscr-flt-val">' . $fval['def'] . '</span></a></li>';
        }

        isc_render_term_li($terms, $fkey, $fval, $selectfield);

    }

    if ( $fval['src'] == 'mk' ) {
        /* taxonomy */
        $selectfield = '';
        $meta_key = $fval['src_key'];

        /* if NOT custom keys selected in admin */
        if ($fval['items_list'] == 'default') {

            global $wpdb;
            if (strpos($meta_key, 'field_') !== false && function_exists('get_field_object')) {
                $field = get_field_object($meta_key);
                if (is_array($field) && isset($field['name'])) {
                    $fieldname = $field['name'];
                    $sql = "SELECT DISTINCT meta_value FROM {$wpdb->prefix}postmeta WHERE meta_key = %s";
                    $terms = $wpdb->get_results($wpdb->prepare($sql, $fieldname), ARRAY_A);
                }
            } else {
                $sql = "SELECT DISTINCT meta_value, meta_value as label FROM {$wpdb->prefix}postmeta WHERE meta_key = %s";
                $terms = $wpdb->get_results($wpdb->prepare($sql, $meta_key), ARRAY_A);
            }

        } elseif ($fval['items_list'] == 'custom') {
            $itemslist = array();
            foreach ($fval['items_in_list'] as $itkey => $itval) {
                $itemslist[$itkey]['meta_value'] = wp_specialchars_decode($itval['key']);
                $itemslist[$itkey]['label'] = (empty($itval['label'])) ? wp_specialchars_decode($itval['key_txt']) : wp_specialchars_decode($itval['label']);
            }

            $terms = $itemslist;
        }

        if (empty($terms)) {
            return false;
        }


        /* add js search */
        /* think reverse for enable disable mini search field. We are asking for disable search in the admin menu */
        if (count($terms) > 5 && !isset($fval['appear']['data']['ed_src'])) {

            if (isset($fval['appear']['data']['ph'])) {
                $placeholder = $fval['appear']['data']['ph'];
            } else {
                $placeholder = '';
            }

            $searchfield = '<input type="text" autocomplete="off" class="isrc-flt-src"  placeholder="' . $placeholder . '">';
            $filter_with_search = true;
        } else {
            $searchfield = '';
        }


        /* set select all with empty value */
        if (isset($fval['def']) && !empty($fval['def'])) {
            $selectfield .= '<li class="isrc-filter-item" data-empty="1" data-filter-text="' . $fval['def'] . '"  data-selectable="yes" data-filter-id="' . $fkey . '" data-src="' . $fval['src'] . '" data-src_key="' . $fval['src_key'] . '" data-value=""><a class="isrc-dropdown-item-link"><span class="isrc-flt-checkbox-ui"></span><span class="iscr-flt-val">' . $fval['def'] . '</span></a></li>';
        }


        foreach ($terms as $key => $val) {
            $val['meta_value'] = wp_specialchars_decode($val['meta_value']);
            $val['label'] = wp_specialchars_decode($val['label']);
            $selectfield .= '<li class="isrc-filter-item" data-filter-text="' . $val['meta_value'] . '"  data-selectable="yes" data-filter-id="' . $fkey . '" data-src="' . $fval['src'] . '" data-src_key="' . $fval['src_key'] . '" data-value="' . $val['meta_value'] . '"><a class="isrc-dropdown-item-link"><span class="isrc-flt-checkbox-ui"></span><span class="iscr-flt-val">' . $val['label'] . '</span></a></li>';
        }

    }

    $css = ' isrc-flt-style_' . $fval['apper'] . ' ';
    $js_src_css = '';
    if ($filter_with_search) {
        $css .= ' isrc-flt-w-src ';
    }

    if (!empty($fval['flttxt'])) {
        $filtertitle = "<div class='isrc-flt-title'><span class='isrc-flt-the-title'>{$fval['flttxt']}</span><i class='isrc-icon icon-down-open'></i></div>";
        $js_src_css = 'w-title';
        $css .= 'w-title2';
    } else {
        $filtertitle = '';
        $css .= 'wo-title2';
    }

    $is_multiselect = ( isset( $fval['multiselect'] ) ) ? '1' : '0';


    if (!empty($searchfield)) {
        return "<div class='isrc-filter-item-wrap isrc_js_search'>{$filtertitle}<div class='isrc-js-src-wrap {$js_src_css}'><label>{$searchfield}<i class='isrc-icon icon-search'></i></label></div><ul class='isrc-group-filter-ul {$css}' data-multiselect='{$is_multiselect}'>{$selectfield}</ul></div>";
    }

    return "<div class='isrc-filter-item-wrap '>{$filtertitle}<ul class='isrc-group-filter-ul {$css}' data-multiselect='{$is_multiselect}'>{$selectfield}</ul></div>";
}

